@@ -20,11 +20,10 @@ TODO:
    (imported modules ? use dis.dis ?)
 
 
-:version:   $Revision: 1.54 $  
 :author:    Sylvain Thenault
-:copyright: 2003-2005 LOGILAB S.A. (Paris, FRANCE)
+:copyright: 2003-2007 LOGILAB S.A. (Paris, FRANCE)
 :contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
-:copyright: 2003-2005 Sylvain Thenault
+:copyright: 2003-2007 Sylvain Thenault
 :contact:   mailto:thenault@gmail.com
 """
 
@@ -41,9 +40,9 @@ from inspect import isdatadescriptor
 from logilab.common.fileutils import norm_read
 from logilab.common.modutils import modpath_from_file
 
-from logilab.astng import nodes
+from logilab.astng import nodes, YES, Instance
 from logilab.astng.utils import ASTWalker
-from logilab.astng._exceptions import ASTNGBuildingException
+from logilab.astng._exceptions import ASTNGBuildingException, InferenceError
 from logilab.astng.raw_building import *
 from logilab.astng.astutils import cvrtr
 
@@ -153,7 +152,10 @@ class ASTNGBuilder:
     """provide astng building methods
     """
     
-    def __init__(self): # XXX _ was the manager, keep for bw compat
+    def __init__(self, manager=None):
+        if manager is None:
+            from logilab.astng import MANAGER as manager
+        self._manager = manager
         self._module = None
         self._file = None
         self._done = None
@@ -161,6 +163,7 @@ class ASTNGBuilder:
         self._metaclass = None        
         self._walker = ASTWalker(self)
         self._dyn_modname_map = {'gtk': 'gtk._gtk'}
+        self._delayed = []
         
     def module_build(self, module, modname=None):
         """build an astng from a living module instance
@@ -186,6 +189,8 @@ class ASTNGBuilder:
         self._module = module
         node = build_module(modname or module.__name__, module.__doc__)
         node.file = node.path = path and abspath(path) or path
+        if self._manager is not None:
+            self._manager._cache[node.file] = self._manager._cache[node.name] = node
         node.package = hasattr(module, '__path__')
         attach___dict__(node)
         self._done = {}
@@ -217,6 +222,7 @@ class ASTNGBuilder:
         finally:
             self._file = None
             sys.path.pop(0)
+        
         return node
     
     def string_build(self, data, modname='', path=None):
@@ -237,7 +243,14 @@ class ASTNGBuilder:
             node.package = path and path.find('__init__.py') > -1 or False
         node.name = modname 
         node.pure_python = True
+        if self._manager is not None:
+            self._manager._cache[node.file] = node
+            if self._file:
+                self._manager._cache[abspath(self._file)] = node
         self._walker.walk(node)
+        while self._delayed:
+            dnode = self._delayed.pop(0)
+            getattr(self, 'delayed_visit_%s' % dnode.__class__.__name__.lower())(dnode)
         return node
 
     # callbacks to build from an existing compiler.ast tree ###################
@@ -442,41 +455,43 @@ class ASTNGBuilder:
             node.parent.set_local(name, node)
         
     def visit_assattr(self, node):
+        """visit a stmt.AssAttr node -> delay it to handle members
+        definition later
+        """
+        self.visit_default(node)
+        self._delayed.append(node)
+    
+    def delayed_visit_assattr(self, node):
         """visit a stmt.AssAttr node -> add name to locals, handle members
         definition
         """
-        self.visit_default(node)
-        frame = node.frame()
-        if isinstance(frame, nodes.Function) and frame.type != 'function':
-            klass = frame.parent.frame()
-            # are we assigning to a (new ?) instance attribute ?
-            try:
-                _self = frame.argnames[0]
-            except IndexError:
-                # first argument is missing !
-                return
-            if isinstance(node.expr, nodes.Name) and node.expr.name == _self:
-                # unittest_scoped_nodes.ClassNodeTC.test_classmethod_attributes
-                #
-                # if frame.type == 'classmethod': XXX at this point we may have
-                # not encountered the classmethod decorator, so we havn't yet
-                # the correct type
-                # hack according to the argument name
-                if _self == 'self': 
-                    iattrs = klass.instance_attrs
+        try:
+            frame = node.frame()
+            for infered in node.expr.infer():
+                if infered is YES:
+                    continue
+                try:
+                    if infered.__class__ is Instance:
+                        infered = infered._proxied
+                        iattrs = infered.instance_attrs
+                    else:
+                        iattrs = infered.locals
+                except AttributeError:
+                    continue
+                values = iattrs.setdefault(node.attrname, [])
+                if node in values:
+                    continue
+                # get assign in __init__ first XXX useful ?
+                if frame.name == '__init__' and values and not \
+                       values[0].frame().name == '__init__':
+                    values.insert(0, node)
                 else:
-                    iattrs = klass.locals
-                # assign if not yet existant in others
-                if not iattrs.has_key(node.attrname):
-                    iattrs[node.attrname] = [node]
-                # but always assign in __init__, except if previous assigment
-                # already come from __init__
-                elif frame.name == '__init__' and not \
-                         iattrs[node.attrname][0].frame().name == '__init__':
-                    iattrs[node.attrname].insert(0, node)
-                else:
-                    iattrs[node.attrname].append(node)
-                    
+                    values.append(node)
+                #print node.attrname, infered, values
+        except InferenceError:
+            #print frame, node
+            pass
+        
     def visit_default(self, node):
         """default visit method, handle the parent attribute
         """
@@ -523,7 +538,7 @@ class ASTNGBuilder:
             if isfunction(member):
                 # verify this is not an imported function
                 if member.func_code.co_filename != modfile:
-                    attach_dummy_node(node, name)
+                    attach_dummy_node(node, name, member)
                     continue
                 object_build_function(node, member)
             elif isbuiltin(member):
@@ -539,7 +554,7 @@ class ASTNGBuilder:
                     continue
                 if member in self._done:
                     class_node = self._done[member]
-                    node.add_local_node(class_node)
+                    node.add_local_node(class_node, name)
                 else:
                     class_node = object_build_class(node, member)
                 # recursion
@@ -554,7 +569,7 @@ class ASTNGBuilder:
                 attach_const_node(node, name, member)
             else:
                 # create an empty node so that the name is actually defined
-                attach_dummy_node(node, name)
+                attach_dummy_node(node, name, member)
 
     def _member_module(self, member):
         modname = getattr(member, '__module__', None)
@@ -571,7 +586,7 @@ def imported_member(node, member, name):
     try:
         getattr(sys.modules[member_module], name)
     except (KeyError, AttributeError):
-        attach_dummy_node(node, name)
+        attach_dummy_node(node, name, member)
     else:
         attach_import_node(node, member_module, name)
     
