@@ -31,11 +31,10 @@ on From and Import :
 
  [1] http://docs.python.org/lib/module-compiler.ast.html
 
-:version:   $Revision: 1.103 $  
 :author:    Sylvain Thenault
-:copyright: 2003-2006 LOGILAB S.A. (Paris, FRANCE)
+:copyright: 2003-2007 LOGILAB S.A. (Paris, FRANCE)
 :contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
-:copyright: 2003-2006 Sylvain Thenault
+:copyright: 2003-2007 Sylvain Thenault
 :contact:   mailto:thenault@gmail.com
 """
 
@@ -67,6 +66,13 @@ except:
     class GenExprInner: 
         """dummy GenExprInner node, shouldn't be used since py < 2.4"""
 
+try:
+    # introduced in python 2.4
+    from compiler.ast import Decorators
+except:
+    class Decorators:
+        """dummy Decorators node, shouldn't be used since py < 2.4"""
+
 from logilab.astng._exceptions import NotFoundError, InferenceError
 from logilab.astng.utils import extend_class
 
@@ -237,9 +243,19 @@ class NodeNG:
                  isinstance(self, (Function, Lambda)) and self is frame):
             return name
         return None
+
+    def eq(self, value):
+        return False
     
 extend_class(Node, NodeNG)
 
+Const.eq = lambda self, value: self.value == value
+
+def decorators_scope(self):
+    # skip the function node to go directly to the upper level scope
+    return self.parent.parent.scope()
+Decorators.scope = decorators_scope
+
 # block range overrides #######################################################
 
 def object_block_range(node, lineno):
