@@ -18,11 +18,10 @@ Each new methods and attributes added on each class are documented
 below.
 
 
-:version:   $Revision: 1.28 $  
 :author:    Sylvain Thenault
-:copyright: 2003-2006 LOGILAB S.A. (Paris, FRANCE)
+:copyright: 2003-2007 LOGILAB S.A. (Paris, FRANCE)
 :contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
-:copyright: 2003-2006 Sylvain Thenault
+:copyright: 2003-2007 Sylvain Thenault
 :contact:   mailto:thenault@gmail.com
 """
 from __future__ import generators
@@ -31,14 +30,15 @@ __doctype__ = "restructuredtext en"
 
 import sys
 
-from logilab.common.compat import chain        
+from logilab.common.compat import chain, set
 
-from logilab.astng._exceptions import NotFoundError, NoDefault, \
-     ASTNGBuildingException, InferenceError
 from logilab.astng.utils import extend_class
-from logilab.astng import YES, MANAGER, Instance, unpack_infer, _infer_stmts, \
+from logilab.astng import YES, MANAGER, Instance, InferenceContext, copy_context, \
+     unpack_infer, _infer_stmts, \
      Class, Const, Dict, Function, GenExpr, Lambda, \
      Module, Name, Pass, Raise, Tuple, Yield
+from logilab.astng import NotFoundError, NoDefault, \
+     ASTNGBuildingException, InferenceError
 
 # module class dict/iterator interface ########################################
     
@@ -58,7 +58,15 @@ class LocalsDictMixIn(object):
     # dictionary of locals with name as key and node defining the local as
     # value    
     locals = None
-    
+
+    def qname(self):
+        """return the 'qualified' name of the node, eg module.name,
+        module.class.name ...
+        """
+        if self.parent is None:
+            return self.name
+        return '%s.%s' % (self.parent.frame().qname(), self.name)
+        
     def frame(self):
         """return the first parent frame node (i.e. Module, Function or Class)
         """
@@ -81,10 +89,12 @@ class LocalsDictMixIn(object):
         
     __setitem__ = set_local
     
-    def add_local_node(self, child_node):
+    def add_local_node(self, child_node, name=None):
         """append a child which should alter locals to the given node"""
-        self._append_node(child_node)
-        self.set_local(child_node.name, child_node)
+        if name != '__class__':
+            # add __class__ node as a child will cause infinite recursion later!
+            self._append_node(child_node)
+        self.set_local(name or child_node.name, child_node)
 
     def _append_node(self, child_node):
         """append a child, linking it in the tree"""
@@ -159,16 +169,20 @@ GenExpr.frame = frame
 
 
 class GetattrMixIn(object):
-    def getattr(self, name, path=None):
+    def getattr(self, name, context=None):
         try:
             return self.locals[name]
         except KeyError:
             raise NotFoundError(name)
         
-    def igetattr(self, name, path=None):
+    def igetattr(self, name, context=None):
         """infered getattr"""
+        # set lookup name since this is necessary to infer on import nodes for
+        # instance
+        context = copy_context(context)
+        context.lookupname = name
         try:
-            return _infer_stmts(self.getattr(name, path), name, frame=self, path=path)
+            return _infer_stmts(self.getattr(name, context), context, frame=self)
         except NotFoundError:
             raise InferenceError(name)
 extend_class(Module, GetattrMixIn)
@@ -197,8 +211,11 @@ class ModuleNG(object):
     # dictionary of globals with name as key and node defining the global
     # as value
     globals = None
-  
-    def getattr(self, name, path=None):
+
+    def pytype(self):
+        return '__builtin__.module'
+    
+    def getattr(self, name, context=None):
         try:
             return self.locals[name]
         except KeyError:
@@ -299,6 +316,11 @@ class FunctionNG(object):
     # arguments are unknown
     argnames = None
 
+    def pytype(self):
+        if 'method' in self.type:
+            return '__builtin__.instancemethod'
+        return '__builtin__.function'
+
     def is_method(self):
         """return true if the function node should be considered as a method"""
         return self.type != 'function'
@@ -358,7 +380,10 @@ class FunctionNG(object):
         if self.argnames is None: # information is missing
             raise NoDefault()
         args, kwargs, last, defaultidx = self._pos_information()
-        i = self.argnames.index(argname)
+        try:
+            i = self.argnames.index(argname)
+        except ValueError:
+            raise NoDefault() # XXX
         if i >= defaultidx and (i - defaultidx) < len(self.defaults):
             return self.defaults[i - defaultidx]
         raise NoDefault()
@@ -368,7 +393,10 @@ class FunctionNG(object):
         a Tuple or Dict instance, else return None
         """
         args, kwargs, last, defaultidx = self._pos_information()
-        i = self.argnames.index(argname)
+        try:
+            i = self.argnames.index(argname)
+        except ValueError:
+            return None # XXX
         if i == last and kwargs:
             valnode = Dict([])
             valnode.parent = self
@@ -402,6 +430,7 @@ Lambda.format_args = FunctionNG.format_args.im_func
 Lambda.default_value = FunctionNG.default_value.im_func
 Lambda.mularg_class = FunctionNG.mularg_class.im_func
 Lambda.type = 'function'
+Lambda.pytype = FunctionNG.pytype.im_func
 
 # Class ######################################################################
 
@@ -426,19 +455,6 @@ def _class_type(klass):
         klass._type = 'class'
     return klass._type
 
-def _class_newstyle(klass):
-    """return a if the given class is new-style or not
-    """
-    if klass._newstyle is not None:
-        return klass._newstyle
-    for base in klass.ancestors(recurs=False):
-        if base.newstyle:
-            klass._newstyle = base.newstyle
-            break
-    if klass._newstyle is None:
-        klass._newstyle = False
-    return klass._newstyle
-
 def _iface_hdlr(iface_node):
     """a handler function used by interfaces to handle suspicious
     interface nodes
@@ -457,10 +473,27 @@ class ClassNG(object):
                     doc="class'type, possible values are 'class' | "
                     "'metaclass' | 'interface' | 'exception'")
     
+    def _newstyle_impl(self, context=None):
+        context = context or InferenceContext()
+        if self._newstyle is not None:
+            return self._newstyle
+        for base in self.ancestors(recurs=False, context=context):
+            if base._newstyle_impl(context):
+                self._newstyle = True
+                break
+        if self._newstyle is None:
+            self._newstyle = False
+        return self._newstyle
+
     _newstyle = None
-    newstyle = property(_class_newstyle,
+    newstyle = property(_newstyle_impl,
                         doc="boolean indicating if it's a new style class"
                         "or not")
+
+    def pytype(self):
+        if self.newstyle:
+            return '__builtin__.type'
+        return '__builtin__.classobj'
     
     # attributes below are set by the builder module or by raw factories
     
@@ -470,7 +503,7 @@ class ClassNG(object):
     # in the class definition)
     basenames = None
 
-    def ancestors(self, recurs=True, path=None):
+    def ancestors(self, recurs=True, context=None):
         """return an iterator on the node base classes in a prefixed
         depth first order
         
@@ -479,17 +512,22 @@ class ClassNG(object):
           ancestors only
         """
         # FIXME: should be possible to choose the resolution order
-        if path is None:
-            path = []
+        # XXX inference make infinite loops possible here (see BaseTransformer
+        # manipulation in the builder module for instance !)
+        context = context or InferenceContext()
         for stmt in self.bases:
             try:
-                for baseobj in stmt.infer(path=path):
+                for baseobj in stmt.infer(context):
                     if not isinstance(baseobj, Class):
                         # duh ?
                         continue
+                    if baseobj is self:
+                        continue # cf xxx above
                     yield baseobj
                     if recurs:
-                        for grandpa in baseobj.ancestors(True, path):
+                        for grandpa in baseobj.ancestors(True, context):
+                            if grandpa is self:
+                                continue # cf xxx above
                             yield grandpa
             except InferenceError:
                 #import traceback
@@ -497,23 +535,23 @@ class ClassNG(object):
                 # XXX log error ?
                 continue
             
-    def local_attr_ancestors(self, name, path=None):
+    def local_attr_ancestors(self, name, context=None):
         """return an iterator on astng representation of parent classes
         which have <name> defined in their locals
         """
-        for astng in self.ancestors(path=path):
+        for astng in self.ancestors(context=context):
             if astng.locals.has_key(name):
                 yield astng
 
-    def instance_attr_ancestors(self, name, path=None):
+    def instance_attr_ancestors(self, name, context=None):
         """return an iterator on astng representation of parent classes
         which have <name> defined in their instance attribute dictionary
         """
-        for astng in self.ancestors(path=path):
+        for astng in self.ancestors(context=context):
             if astng.instance_attrs.has_key(name):
                 yield astng
 
-    def local_attr(self, name, path=None):
+    def local_attr(self, name, context=None):
         """return the astng associated to name in this class locals or
         in its parents
 
@@ -525,11 +563,11 @@ class ClassNG(object):
             return self[name]
         except KeyError:
             # get if from the first parent implementing it if any
-            for class_node in self.local_attr_ancestors(name, path):
+            for class_node in self.local_attr_ancestors(name, context):
                 return class_node[name]
         raise NotFoundError(name)
         
-    def instance_attr(self, name, path=None):
+    def instance_attr(self, name, context=None):
         """return the astng nodes associated to name in this class instance
         attributes dictionary or in its parents
 
@@ -541,11 +579,11 @@ class ClassNG(object):
             return self.instance_attrs[name]
         except KeyError:
             # get if from the first parent implementing it if any
-            for class_node in self.instance_attr_ancestors(name, path):
+            for class_node in self.instance_attr_ancestors(name, context):
                 return class_node.instance_attrs[name]
         raise NotFoundError(name)
 
-    def getattr(self, name, path=None):
+    def getattr(self, name, context=None):
         """this method doesn't look in the instance_attrs dictionary since it's
         done by an Instance proxy at inference time.
         
@@ -559,24 +597,28 @@ class ClassNG(object):
         # XXX need proper meta class handling + MRO implementation
         if name == '__mro__':
             return tuple(self.ancestors(recurs=True))
-        for classnode in self.ancestors(recurs=False, path=path):
+        for classnode in self.ancestors(recurs=False, context=context):
             try:
-                return classnode.getattr(name, path)
+                return classnode.getattr(name, context)
             except NotFoundError:
                 continue
         raise NotFoundError(name)
 
-    def igetattr(self, name, path=None):
+    def igetattr(self, name, context=None):
         """infered getattr, need special treatment in class to handle
         descriptors
         """
+        # set lookoup name since this is necessary to infer on import nodes for
+        # instance
+        context = copy_context(context)
+        context.lookupname = name
         try:
-            for infered in _infer_stmts(self.getattr(name, path), name,
-                                        frame=self, path=path):
+            for infered in _infer_stmts(self.getattr(name, context), context,
+                                        frame=self):
                 # yield YES object instead of descriptors when necessary
                 if not isinstance(infered, Const) and isinstance(infered, Instance):
                     try:
-                        infered._proxied.getattr('__get__', path)
+                        infered._proxied.getattr('__get__', context)
                     except NotFoundError:
                         yield infered
                     else:
@@ -584,13 +626,13 @@ class ClassNG(object):
                 else:
                     yield infered
         except NotFoundError:
-            if not name.startswith('__') and self.has_dynamic_getattr(path):
+            if not name.startswith('__') and self.has_dynamic_getattr(context):
                 # class handle some dynamic attributes, return a YES object
                 yield YES
             else:
                 raise InferenceError(name)
         
-    def has_dynamic_getattr(self, path=None):
+    def has_dynamic_getattr(self, context=None):
         """return True if the class has a custom __getattr__ or
         __getattribute__ method
         """
@@ -598,11 +640,12 @@ class ClassNG(object):
         if self.name == 'Values' and self.root().name == 'optparse':
             return True
         try:
-            self.getattr('__getattr__', path)
+            self.getattr('__getattr__', context)
             return True
         except NotFoundError:
+            #if self.newstyle: XXX cause an infinite recursion error
             try:
-                getattribute = self.getattr('__getattribute__', path)[0]
+                getattribute = self.getattr('__getattribute__', context)[0]
                 if getattribute.root().name != '__builtin__':
                     # class has a custom __getattribute__ defined
                     return True
