@@ -11,14 +11,11 @@
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """tests for the astng builder module
-
-Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
-http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 
 import unittest
 import sys
-from os.path import join, abspath
+from os.path import join, abspath, dirname
 
 from logilab.common.testlib import TestCase, unittest_main
 from unittest_inference import get_name_node
@@ -159,6 +156,7 @@ class BuilderTC(TestCase):
         self.assert_('mode' in fclass)
         self.assert_('read' in fclass)
         self.assert_(fclass.newstyle)
+        self.assert_(fclass.pytype(), '__builtin__.type')
         self.assert_(isinstance(fclass['read'], nodes.Function))
         self.assert_(isinstance(fclass['__doc__'], nodes.Const), fclass['__doc__'])
         # check builtin function has argnames == None
@@ -167,6 +165,10 @@ class BuilderTC(TestCase):
         # just check type and object are there
         builtin_astng.getattr('type')
         builtin_astng.getattr('object')
+        # check open file alias
+        builtin_astng.getattr('open')
+        # check 'help' is there (defined dynamically by site.py)
+        builtin_astng.getattr('help')
         # check property has __init__
         pclass = builtin_astng['property']
         self.assert_('__init__' in pclass)
@@ -207,7 +209,29 @@ class BuilderTC(TestCase):
         self.assert_('errno' in container)
         self.assert_('strerror' in container)
         self.assert_('filename' in container)
+
+    def test_inspect_build_type_object(self):
+        import __builtin__
+        builtin_astng = self.builder.inspect_build(__builtin__)
+        
+        infered = list(builtin_astng.igetattr('object'))
+        self.assertEquals(len(infered), 1)
+        infered = infered[0]
+        self.assertEquals(infered.name, 'object')
+        try:
+            infered.as_string()
+        except:
+            print repr(infered)
             
+        infered = list(builtin_astng.igetattr('type'))
+        self.assertEquals(len(infered), 1)
+        infered = infered[0]
+        self.assertEquals(infered.name, 'type')
+        try:
+            infered.as_string()
+        except:
+            print repr(infered)
+        
     def test_package_name(self):
         """test base properties and method of a astng module"""
         datap = self.builder.file_build('data/__init__.py', 'data')
@@ -439,7 +463,22 @@ class ModuleBuildTC(FileBuildTC):
         self.module = abuilder.module_build(test_module)
 
 
-__all__ = ('BuilderModuleBuildTC', 'BuilderFileBuildTC', 'BuilderTC')
+class InferedBuildTC(TestCase):
+
+    def test(self):
+        from logilab import astng
+        import compiler
+        sn = astng.MANAGER.astng_from_file(join(astng.__path__[0], 'scoped_nodes.py'))
+        astastng = astng.MANAGER.astng_from_file(join(compiler.__path__[0], 'ast.py'))
+        # check monkey patching of the compiler module has been infered
+        lclass = list(astastng.igetattr('Lambda'))
+        self.assertEquals(len(lclass), 1)
+        lclass = lclass[0]
+        self.assert_('as_string' in lclass.locals, lclass.locals.keys())
+        self.assert_('format_args' in lclass.locals, lclass.locals.keys())
+        self.assert_('type' in lclass.locals.keys())
+        
+#__all__ = ('BuilderModuleBuildTC', 'BuilderFileBuildTC', 'BuilderTC')
 
 if __name__ == '__main__':
     unittest_main()
