@@ -11,17 +11,13 @@
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """tests for the astng inference capabilities
-
-Copyright (c) 2005-2006 LOGILAB S.A. (Paris, FRANCE).
-http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 
 import sys
-from os.path import join, abspath
-
-from logilab.astng import builder, nodes, inference, utils, YES, Instance
 from logilab.common.testlib import TestCase, unittest_main
 
+from logilab.astng import builder, nodes, inference, YES, Instance
+
 def get_name_node(start_from, name, index=0):
     return [n for n in start_from.nodes_of_class(nodes.Name) if n.name == name][index]
 
@@ -96,7 +92,7 @@ a, b= b, a # Gasp !
         self.failUnlessEqual(obj.root().name, __name__)
         self.failUnlessRaises(StopIteration, infered.next)
 
-    def test_name_inference(self):
+    def test_builtin_name_inference(self):
         infered = self.astng['C']['meth1']['var'].infer()
         var = infered.next()
         self.failUnlessEqual(var.name, 'object')
@@ -181,7 +177,7 @@ a, b= b, a # Gasp !
         self.failUnlessRaises(StopIteration, infered.next)
         
     def test_getattr_inference1(self):
-        infered = self.astng['ex'].infer(path=[])
+        infered = self.astng['ex'].infer()
         exc = infered.next()
         self.failUnless(isinstance(exc, inference.Instance))
         self.failUnlessEqual(exc.name, 'Exception')
@@ -228,7 +224,7 @@ a, b= b, a # Gasp !
         self.failUnlessRaises(StopIteration, infered.next)
 
     def test_args_default_inference2(self):
-        infered = self.astng['C']['meth3']['d'].infer(name='d')
+        infered = self.astng['C']['meth3'].ilookup('d')
         obj1 = infered.next()
         self.failUnless(isinstance(obj1, nodes.Const))
         self.failUnlessEqual(obj1.value, 4)
@@ -295,7 +291,6 @@ a = f()
         a = astng['a']
         a_infer = a.infer()
         self.failUnlessEqual(a_infer.next().value, 1)
-        self.failUnlessEqual(a_infer.next(), YES)
         self.failUnlessRaises(StopIteration, a_infer.next)
         
     def test_exc_ancestors(self):
@@ -454,7 +449,7 @@ xxx = load_module_from_name('__pkginfo__')
         xxx = astng['xxx']
         infered = list(xxx.infer())
         self.failUnlessEqual([n.__class__ for n in infered],
-                             [nodes.Const, YES.__class__, YES.__class__])
+                             [nodes.Const, YES.__class__])
 
     def test_method_argument(self):
         data = '''
@@ -562,7 +557,247 @@ def no_conjugate_member(magic_flag):
                              [1.0, 1.0j])
         self.failUnlessEqual([i.value for i in get_name_node(astng, 'something', -1).infer()],
                              [1.0, 1.0j])
+
+    def test_simple_subscript(self):
+        data = '''
+a = [1, 2, 3][0]
+b = (1, 2, 3)[1]
+c = (1, 2, 3)[-1]
+d = a + b + c
+print d
+        '''
+        astng = builder.string_build(data, __name__, __file__)
+        self.failUnlessEqual([i.value for i in get_name_node(astng, 'a', -1).infer()],
+                             [1])
+        self.failUnlessEqual([i.value for i in get_name_node(astng, 'b', -1).infer()],
+                             [2])
+        self.failUnlessEqual([i.value for i in get_name_node(astng, 'c', -1).infer()],
+                             [3])
+        # kill me
+        #self.failUnlessEqual([i.value for i in get_name_node(astng, 'd', -1).infer()],
+        #                     [6])
+
+    def test_simple_for(self):
+        data = '''
+for a in [1, 2, 3]:
+    print a
+for b,c in [(1,2), (3,4)]:
+    print b
+    print c
+
+print [(d,e) for e,d in ([1,2], [3,4])]
+        '''
+        astng = builder.string_build(data, __name__, __file__)
+        self.failUnlessEqual([i.value for i in get_name_node(astng, 'a', -1).infer()],
+                             [1, 2, 3])
+        self.failUnlessEqual([i.value for i in get_name_node(astng, 'b', -1).infer()],
+                             [1, 3])
+        self.failUnlessEqual([i.value for i in get_name_node(astng, 'c', -1).infer()],
+                             [2, 4])
+        self.failUnlessEqual([i.value for i in get_name_node(astng, 'd', -1).infer()],
+                             [2, 4])
+        self.failUnlessEqual([i.value for i in get_name_node(astng, 'e', -1).infer()],
+                             [1, 3])
+
+
+    def test_simple_for_genexpr(self):
+        if sys.version_info < (2, 4):
+            return
+        data = '''
+print ((d,e) for e,d in ([1,2], [3,4]))
+        '''
+        astng = builder.string_build(data, __name__, __file__)
+        self.failUnlessEqual([i.value for i in get_name_node(astng, 'd', -1).infer()],
+                             [2, 4])
+        self.failUnlessEqual([i.value for i in get_name_node(astng, 'e', -1).infer()],
+                             [1, 3])
+
+
+    def test_builtin_help(self):
+        data = '''
+help()
+        '''
+        astng = builder.string_build(data, __name__, __file__)
+        node = get_name_node(astng, 'help', -1)
+        infered = list(node.infer())
+        self.failUnlessEqual(len(infered), 1)
+        self.assertIsInstance(infered[0], Instance)
+        self.failUnlessEqual(str(infered[0]),
+                             'Instance of site._Helper')
+        
+    def test_builtin_open(self):
+        data = '''
+open("toto.txt")
+        '''
+        astng = builder.string_build(data, __name__, __file__)
+        node = get_name_node(astng, 'open', -1)
+        infered = list(node.infer())
+        self.failUnlessEqual(len(infered), 1)
+        if open is file:
+            # On python < 2.5 open and file are the same thing.
+            self.assertIsInstance(infered[0], nodes.Class)
+            self.failUnlessEqual(infered[0].name, 'file')
+        else:
+            # On python >= 2.5 open is a builtin function.
+            self.assertIsInstance(infered[0], nodes.Function)
+            self.failUnlessEqual(infered[0].name, 'open')
                 
+    def test_callfunc_context_inference(self):
+        data = '''
+def mirror(arg=None):
+    return arg
+
+un = mirror(1)
+        '''
+        astng = builder.string_build(data, __name__, __file__)
+        infered = list(astng.igetattr('un'))
+        self.failUnlessEqual(len(infered), 1)
+        self.assertIsInstance(infered[0], nodes.Const)
+        self.failUnlessEqual(infered[0].value, 1)
+                
+    def test_callfunc_context_inference_lambda(self):
+        data = '''
+mirror = lambda x=None: x
+
+un = mirror(1)
+        '''
+        astng = builder.string_build(data, __name__, __file__)
+        infered = list(astng.igetattr('un'))
+        self.failUnlessEqual(len(infered), 1)
+        self.assertIsInstance(infered[0], nodes.Const)
+        self.failUnlessEqual(infered[0].value, 1)
+        
+    def test_factory_method(self):
+        if sys.version_info < (2, 4):
+            self.skip('this test require python >= 2.4')
+        data = '''
+class Super(object):
+      @classmethod
+      def instance(cls):
+              return cls()
+
+class Sub(Super):
+      def method(self):
+              print 'method called'
+
+sub = Sub.instance()
+        '''
+        astng = builder.string_build(data, __name__, __file__)
+        infered = list(astng.igetattr('sub'))
+        self.failUnlessEqual(len(infered), 1)
+        self.assertIsInstance(infered[0], Instance)
+        self.failUnlessEqual(infered[0]._proxied.name, 'Sub')
+        
+    def test_base_operator(self):
+        data = '''
+a = "*" * 80
+b = 1 / 2.
+c = b - 1
+d = [[]]*3
+
+class myarray:
+    def __init__(self, array):
+        self.array = array
+
+    def __mul__(self, x):
+        return myarray([2,4,6])
+
+    def astype(self):
+        return "ASTYPE"
+
+def randint(maximum):
+    if maximum is not None:
+        return myarray([1,2,3]) * 2
+    else:
+        return int(5)
+
+x = randint(1)
+        '''
+        astng = builder.string_build(data, __name__, __file__)
+        # a
+        infered = list(astng.igetattr('a'))
+        self.failUnlessEqual(len(infered), 1)
+        value = infered[0]
+        self.assertIsInstance(value, nodes.Const)
+        self.failUnlessEqual(value._proxied.name, 'str')
+        # hey...
+        self.failUnlessEqual(value.value, '********************************************************************************')
+        # b
+        infered = list(astng.igetattr('b'))
+        self.failUnlessEqual(len(infered), 1)
+        value = infered[0]
+        self.assertIsInstance(value, nodes.Const)
+        self.failUnlessEqual(value._proxied.name, 'float')
+        self.failUnlessEqual(value.value, 1/2.)
+        # c
+        infered = list(astng.igetattr('c'))
+        self.failUnlessEqual(len(infered), 1)
+        value = infered[0]
+        self.assertIsInstance(value, nodes.Const)
+        self.failUnlessEqual(value._proxied.name, 'float')
+        self.failUnlessEqual(value.value, 1/2.-1)
+        # d
+        infered = list(astng.igetattr('d'))
+        self.failUnlessEqual(len(infered), 1, infered)
+        value = infered[0]
+        self.assertIsInstance(value, nodes.List)
+        # x
+        infered = list(astng.igetattr('x'))
+        self.failUnlessEqual(len(infered), 2)
+        value = [str(v) for v in infered]
+        # The __name__ trick here makes it work when invoked directly
+        # (__name__ == '__main__') and through pytest (__name__ ==
+        # 'unittest_inference')
+        self.assertEquals(value, ['Instance of %s.myarray' % (__name__,),
+                                  'Instance of __builtin__.int'])
+
+        
+    def test_import_as(self):
+        data = '''
+import os.path as osp
+print osp.dirname(__file__)
+
+from os.path import exists as e
+assert e(__file__)
+
+from new import code as make_code
+print make_code
+        '''
+        astng = builder.string_build(data, __name__, __file__)
+        infered = list(astng.igetattr('osp'))
+        self.failUnlessEqual(len(infered), 1)
+        self.assertIsInstance(infered[0], nodes.Module)
+        self.failUnlessEqual(infered[0].name, 'os.path')
+        infered = list(astng.igetattr('e'))
+        self.failUnlessEqual(len(infered), 1)
+        self.assertIsInstance(infered[0], nodes.Function)
+        self.failUnlessEqual(infered[0].name, 'exists')
+        infered = list(astng.igetattr('make_code'))
+        self.failUnlessEqual(len(infered), 1)
+        self.assertIsInstance(infered[0], Instance)
+        self.failUnlessEqual(str(infered[0]), 'Instance of __builtin__.type')
+
+    def test_nonregr_lambda_arg(self):
+        data = '''
+def f(g = lambda: None):
+        g().x
+'''
+        astng = builder.string_build(data, __name__, __file__)
+        callfuncnode = astng['f'].code.nodes[0].expr.expr
+        infered = list(callfuncnode.infer())
+        self.failUnlessEqual(len(infered), 1)
+        self.assertIsInstance(infered[0], nodes.Const)
+        self.failUnlessEqual(infered[0].value, None)
+
+    def test_nonregr_getitem_empty_tuple(self):
+        data = '''
+def f(x):
+        a = ()[x]
+        '''
+        astng = builder.string_build(data, __name__, __file__)
+        infered = list(astng['f'].ilookup('a'))
+        self.failUnlessEqual(len(infered), 1)
+        self.failUnlessEqual(infered[0], YES)
+        
 if __name__ == '__main__':
-    from logilab.common.testlib import unittest_main
     unittest_main()
